# Generated by Django 2.2.25 on 2021-12-20 16:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_camunda", "0004_camundaconfig_enabled"),
    ]

    operations = [
        migrations.AlterField(
            model_name="camundaconfig",
            name="enabled",
            field=models.BooleanField(
                default=True,
                help_text="Global flag to enable/disable Camunda integration.",
                verbose_name="enabled",
            ),
        ),
    ]
